#include <stdio.h>

#include "http.h"
#include "http_loop.h"

http_loop_gctx *http_loop_gctx_init() {
  http_loop_gctx *gctx = calloc(sizeof(http_loop_gctx), 1);

  return gctx;
}

http_loop_ctx *http_loop_ctx_init(http_loop_gctx *g) {
  http_loop_ctx *ctx = calloc(sizeof(http_loop_ctx), 1);
  ctx->g = g;

  return ctx;
}

void http_loop_ctx_free(http_loop_ctx *ctx) {
  http_loop_ctx_reset(ctx);

  free(ctx);
}

void http_loop_ctx_reset(http_loop_ctx *ctx) {
  ctx->route = NULL;
  ctx->current_step = 0;

  if (ctx->req.body != NULL) {
    free((void *)ctx->req.body);
    ctx->req.body = NULL;
  }

  if (ctx->res.head != NULL) {
    free((void *)ctx->res.head);
    ctx->res.head = NULL;
  }

  if (ctx->res.body != NULL) {
    if (ctx->res.body_type == http_response_body_buf && ctx->res.owns_body) {
      free((void *)ctx->res.body);
    } else if (ctx->res.body_type == http_response_body_file) {
      fclose(ctx->res.body);
    }
  }

  ctx->res.body = NULL;

  for (size_t i = 0; i < ctx->res.header_count; i++) {
    if (ctx->res.headers[i].owned) {
      free((void *)ctx->res.headers[i].value);
    }
  }

  ctx->res.header_count = 0;

  ctx->res.status = 0;
  ctx->res.head_len = 0;
  ctx->res.head_written = 0;
  ctx->res.body_len = 0;
  ctx->res.body_written = 0;
  ctx->res.owns_body = false;
}
