#include <stdio.h>

#include "lander.h"

static const char index_page[] =
    "<!DOCTYPE html>\n"
    "<html>\n"
    "  <body>\n"
    "    <h1>r8r.be</h1>\n"
    "    <p>This is the URL shortener and pastebin accompanying my site, <a "
    "href=\"https://rustybever.be\">The Rusty Bever</a>.</p>\n"
    "  </body>\n"
    "</html>\n";

bool lander_get_index(event_loop_conn *conn) {
  http_loop_ctx *ctx = conn->ctx;

  http_loop_res_set_body_buf(ctx, index_page, sizeof(index_page) - 1, false);

  conn->state = event_loop_conn_state_res;
  return true;
}

bool lander_get_entry(event_loop_conn *conn) {
  http_loop_ctx *ctx = conn->ctx;

  const char *key = &ctx->req.path[ctx->req.regex_groups[1].rm_so];
  int key_len = ctx->req.regex_groups[1].rm_eo - ctx->req.regex_groups[1].rm_so;

  Entry *entry;
  TrieExitCode res = trie_search_len(ctx->g->trie, &entry, key, key_len);

  if (res == NotFound) {
    ctx->res.status = http_not_found;
  } else if (entry->type == Redirect) {
    ctx->res.status = http_moved_permanently;
    http_loop_res_add_header(ctx, http_header_location, entry->string, false);
  } else if (entry->type == Paste) {
    char fname[8 + key_len];
    sprintf(fname, "pastes/%.*s", key_len, key);

    http_loop_res_set_body_file(ctx, fname);
  }

  conn->state = event_loop_conn_state_res;

  return true;
}

http_route lander_routes[] = {
    {.type = http_route_literal,
     .method = http_get,
     .path = "/",
     .steps = {lander_get_index, NULL}},
    {.type = http_route_regex,
     .method = http_get,
     .path = "^/\\([^/]\\+\\)$",
     .steps = {lander_get_entry, NULL}},
    {.type = http_route_literal,
     .method = http_post,
     .path = "/s/",
     .steps = {http_loop_step_auth, http_loop_step_body_to_buf,
               lander_post_redirect, NULL}}};
